// fairCTF.cpp : Defines the entry point for the DLL application.
//

#include "bzfsAPI.h"

BZ_GET_PLUGIN_VERSION

class fairCTF : public bz_EventHandler
{
public:
	fairCTF();
	virtual bool isEven();
	virtual void process (bz_EventData *eventData);

	bool is_even;
};


fairCTF fairctf;


BZF_PLUGIN_CALL int bz_Load ( const char* /*commandLine*/ )
{

	bz_registerEvent (bz_eTickEvent, &fairctf);

  bz_debugMessage(4,"fairCTF plugin loaded");
  return 0;
}

BZF_PLUGIN_CALL int bz_Unload ( void )
{

	bz_removeEvent (bz_eTickEvent, &fairctf);

  bz_debugMessage(4,"fairCTF plugin unloaded");
  return 0;
}

fairCTF::fairCTF()
{
	is_even = true;
}

bool fairCTF::isEven()
{

	int teamsizes[4]; 

	teamsizes[0] = bz_getTeamCount (eRedTeam);
	teamsizes[1] = bz_getTeamCount (eGreenTeam);
	teamsizes[2] = bz_getTeamCount (eBlueTeam);
	teamsizes[3] = bz_getTeamCount (ePurpleTeam);

	int smallestTeam = 10000; //impossibly high
	int largestTeam = 0;

	for (int x = 0; x < 4; x++)
	{
		if (teamsizes[x] > largestTeam)
		{
			largestTeam = teamsizes[x];
		}
		if (teamsizes[x] != 0 && teamsizes[x] < smallestTeam)
		{
			smallestTeam = teamsizes[x];
		}
	}

	//check differences and ratios

	if (smallestTeam == 10000 || largestTeam == smallestTeam) //equal, or server has no team tanks
	{
		return true;
	}
	if (smallestTeam <= 2) // 2 vs 3+ is not fair
	{
		return false;
	}
	if (largestTeam - smallestTeam == 1) //after 2v3, a one-player gap is acceptible
	{
		return true;
	}
	if ((static_cast<float> (largestTeam - smallestTeam)) / smallestTeam >= 0.25) //greater than 25% gap
	{
		return false;
	}
	if (largestTeam - smallestTeam >= 4) //off by 4 or more
	{
		return false;
	}

	return true; 
}

void fairCTF::process(bz_EventData *eventData)
{
	if (isEven())
	{
		if (!is_even)
		{
			bz_sendTextMessage(BZ_SERVER, BZ_ALLUSERS, "Teams are even enough to be fair. CTF enabled.");
			is_even = true;
		}
	}
	else
	{
		if (is_even)
		{
			bz_sendTextMessage(BZ_SERVER, BZ_ALLUSERS, "Teams are not even. CTF disabled.");
			is_even = false;
		}

		bzAPIIntList pl;

		bz_getPlayerIndexList(&pl);
		
		for (unsigned int x = 0; x < pl.size(); x++)
		{

			bz_PlayerRecord *pr;
			pr = bz_getPlayerByIndex(pl.get(x));

			if (pr != NULL)
			{
				if (pr->currentFlag == "Red team flag" || pr->currentFlag == "Green team flag" || pr->currentFlag == "Blue team flag" || pr->currentFlag == "Purple team flag")
				{
					bz_removePlayerFlag(pl.get(x));
					bz_sendTextMessage (BZ_SERVER, pl.get(x), "No CTF!");
				}
			}

			bz_freePlayerRecord (pr);
		}
	}
}











// Local Variables: ***
// mode:C++ ***
// tab-width: 8 ***
// c-basic-offset: 2 ***
// indent-tabs-mode: t ***
// End: ***
// ex: shiftwidth=2 tabstop=8

